@@ -196,7 +196,7 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
   switch(attr(socket, "protocol"),
          req =,
          rep = {
-           nano[["context_open"]] <- function() {
+           nano[["context_open"]] <- function(verify = TRUE) {
              if (is.null(sock2)) sock2 <<- socket
              nano[["context_close"]] <- function() if (length(sock2)) {
                r <- close(socket)
@@ -205,11 +205,11 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
                rm(list = c("context", "context_close"), envir = nano)
                r
              }
-            socket <<- nano[["context"]] <- context(sock2)
+            socket <<- nano[["context"]] <- context(sock2, verify = verify)
            }
          },
          sub = {
-           nano[["context_open"]] <- function() {
+           nano[["context_open"]] <- function(verify = TRUE) {
              if (is.null(sock2)) sock2 <<- socket
              nano[["context_close"]] <- function() if (length(sock2)) {
                r <- close(socket)
@@ -218,7 +218,7 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
                rm(list = c("context", "context_close"), envir = nano)
                r
              }
-             socket <<- nano[["context"]] <- context(sock2)
+             socket <<- nano[["context"]] <- context(sock2, verify = verify)
            }
            nano[["subscribe"]] <- function(topic = NULL)
              subscribe(socket, topic = topic)
@@ -226,7 +226,7 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
              unsubscribe(socket, topic = topic)
          },
          surveyor = {
-           nano[["context_open"]] <- function() {
+           nano[["context_open"]] <- function(verify = TRUE) {
              if (is.null(sock2)) sock2 <<- socket
              nano[["context_close"]] <- function() if (length(sock2)) {
                r <- close(socket)
@@ -235,13 +235,13 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
                rm(list = c("context", "context_close"), envir = nano)
                r
              }
-             socket <<- nano[["context"]] <- context(sock2)
+             socket <<- nano[["context"]] <- context(sock2, verify = verify)
            }
            nano[["survey_time"]] <- function(value = 1000L)
              survey_time(socket, value = value)
          },
          respondent = {
-           nano[["context_open"]] <- function() {
+           nano[["context_open"]] <- function(verify = TRUE) {
              if (is.null(sock2)) sock2 <<- socket
              nano[["context_close"]] <- function() if (length(sock2)) {
                r <- close(socket)
@@ -250,7 +250,7 @@ nano <- function(protocol = c("bus", "pair", "push", "pull", "pub", "sub",
                rm(list = c("context", "context_close"), envir = nano)
                r
              }
-             socket <<- nano[["context"]] <- context(sock2)
+             socket <<- nano[["context"]] <- context(sock2, verify = verify)
            }
          },
          NULL)
@@ -408,15 +408,6 @@ print.conditionVariable <- function(x, ...) {
 
 }
 
-#' @export
-#'
-print.nanoPipe <- function(x, ...) {
-
-  cat(sprintf("< nanoPipe >\n - id: %d\n", attr(x, "id")), file = stdout())
-  invisible(x)
-
-}
-
 #' @export
 #'
 `[[.nano` <- function(x, i, exact = FALSE)
@@ -444,10 +435,18 @@ print.nanoPipe <- function(x, ...) {
 #'
 `$<-.recvAio` <- function(x, name, value) x
 
+#' @export
+#'
+`[[<-.recvAio` <- function(x, i, value) x
+
 #' @export
 #'
 `$<-.sendAio` <- function(x, name, value) x
 
+#' @export
+#'
+`[[<-.sendAio` <- function(x, i, value) x
+
 #' @export
 #'
 .DollarNames.nano <- function(x, pattern = "")
