@@ -549,7 +549,7 @@ SEXP rnng_stream_listen(SEXP url, SEXP textframes, SEXP pem) {
 SEXP rnng_stream_close(SEXP stream) {
 
   if (R_ExternalPtrTag(stream) != nano_StreamSymbol)
-    Rf_error("'stream' is not a valid/active Stream");
+    Rf_error("'stream' is not a valid or active Stream");
   nng_stream *sp = (nng_stream *) R_ExternalPtrAddr(stream);
   nng_stream_free(sp);
   R_SetExternalPtrTag(stream, R_NilValue);
