@@ -67,7 +67,6 @@ raio <- n$recv_aio(timeout = 1L)
 nanotestp(raio)
 nanotest(is_error_value(call_aio(raio)$data))
 nanotest(is_error_value(raio$data))
-nanotesterr(msg_pipe(raio), "successful Aio")
 nanotestz(n$send(data.frame(), block = FALSE))
 nanotest(is.list(n1$recv(block = FALSE, keep.raw = TRUE)))
 nanotestz(n1$send("test", mode = "raw", block = 500))
@@ -83,10 +82,12 @@ nanotesterr(n$send("wrong mode", mode = "none"), "mode")
 raio <- n1$recv_aio(timeout = 500, keep.raw = TRUE)
 nanotestp(raio)
 raio$newfield <- "doesnotwork"
+raio[["newfield"]] <- "doesnotwork"
 nanotestn(raio$newfield)
 nanotest(is_aio(raio))
 saio <- n$send_aio(c(1.1, 2.2), mode = "raw", timeout = 500)
 saio$newfield <- "doesnotwork"
+saio[["newfield"]] <- "doesnotwork"
 nanotestn(saio$newfield)
 nanotest(is_aio(saio))
 nanotest(is.logical(.unresolved(saio)))
@@ -94,21 +95,18 @@ nanotest(is.logical(unresolved(saio)))
 nanotestnn(raio$raw)
 nanotest(identical(unserialize(call_aio(raio)$raw), raio$data))
 msg <- n1$recv_aio(mode = "numer", keep.raw = TRUE, timeout = 500)
+nanotest(identical(call_aio(msg), msg))
+msg <- n1$recv_aio(mode = "complex", keep.raw = TRUE, timeout = 500)
 nanotestn(stop_aio(msg))
 nanotestn(stop_aio(n))
 nanotest(identical(call_aio(msg), msg))
 nanotesterr(msg$raw, "active")
 nanotesterr(msg$data, "active")
-nanotesterr(msg_pipe(msg), "active")
 nanotest(identical(call_aio(n), n))
 nanotest(!is_aio(n))
 sraio <- n$send_aio(as.raw(0L), mode = "r", timeout = 500)
 rraio <- n1$recv_aio(mode = "r", timeout = 500)
 nanotestnn(call_aio(rraio)$data)
-nanotestxp(p <- msg_pipe(rraio))
-nanotest(is.integer(attr(p, "id")))
-nanotestp(p)
-nanotestz(close(p))
 sraio <- n$send_aio(c(1+2i, 4+3i), mode = "ra", timeout = 500)
 rraio <- n1$recv_aio(mode = "co", timeout = 500)
 nanotestnn(call_aio(rraio)$data)
@@ -172,7 +170,7 @@ nanotestn(stat(rep, "nonexistentstat"))
 nanotestnano(req$opt("req:resend-time", 1000))
 nanotest(req$opt("req:resend-time") == 1000L)
 nanotesterr(req$opt("none"), "supported")
-nanotestxp(req$context_open())
+nanotestxp(req$context_open(verify = FALSE))
 nanotest(inherits(req$context, "nanoContext"))
 nanotest(inherits(req$context, "nano"))
 nanotest(is.integer(req$context$id))
@@ -293,7 +291,6 @@ nanotest(inherits(respond <- nano(protocol = "respondent"), "nanoObject"))
 nanotestnano(bus)
 nanotestw(listen(bus, url = "test") == 3L)
 nanotestw(dial(bus, url = "test") == 3L)
-nanotestxp(weakref(push) <- pull)
 nanotestz(close(bus))
 nanotestz(close(push))
 nanotestz(close(pull))
@@ -304,6 +301,8 @@ nanotesterr(socket(dial = "test"), "argument")
 nanotesterr(socket(listen = "test"), "argument")
 
 nanotestnn(ncurl("http://www.cam.ac.uk/"))
+nanotestnn(ncurl("http://www.cam.ac.uk/", follow = FALSE, response = "date"))
+nanotestnn(ncurl("http://www.cam.ac.uk/", follow = FALSE, response = list("date")))
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = TRUE))
 nanotestnn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server"))
 nanotestnn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server")))
@@ -325,10 +324,10 @@ nanotest(is_error_value(ncurl("http", async = TRUE)$data))
 haio <- ncurl("https://i.i", async = TRUE)
 nanotest(is_error_value(call_aio(haio)$raw))
 nanotestp(haio$data)
-sess <- ncurl_session("https://httpbin.org/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 1000L)
+sess <- ncurl_session("https://httpbin.org/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 2000L)
 nanotestnn(sess)
 nanotest(is_error_value(sess) || length(transact(sess)) == 4L)
-sess <- ncurl_session("https://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 1000)
+sess <- ncurl_session("https://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 2000)
 nanotestnn(sess)
 nanotest(is_error_value(sess) || length(transact(sess)) == 4L)
 nanotest(is_error_value(sess) || close(sess) == 0L)
@@ -365,9 +364,9 @@ if (length(s)) {
   nanotest(is.integer(send(s, "message1", block = 500L)))
   nanotestnn(recv(s, block = FALSE))
   nanotest(is.integer(send(s, "message2", block = FALSE)))
-  nanotestnn(recv(s, block = 100))
+  nanotestnn(recv(s, mode = 2L, block = 100))
   nanotest(is.integer(send(s, 2L, block = 500)))
-  nanotest(is_aio(sr <- recv_aio(s, mode = "integer", keep.raw = TRUE, timeout = 500L)))
+  nanotest(is_aio(sr <- recv_aio(s, mode = "integer", keep.raw = TRUE, timeout = 500L, n = 8192L)))
   nanotestnn(call_aio(sr)[["raw"]])
   nanotestn(stop_aio(sr))
   nanotest(is_aio(ss <- send_aio(s, "async", timeout = 500L)))
@@ -379,6 +378,7 @@ if (length(s)) {
   nanotest(cv_value(cv) > 0L)
   nanotest(!wait(cv))
   nanotest(is.character(opt(s, "ws:request-headers")))
+  nanotestnn(opt(s, "tcp-nodelay") <- FALSE)
   nanotesterr(recv(s, mode = "none", block = FALSE), "mode")
   nanotesterr(recv(s, mode = "c", block = FALSE), "mode")
   nanotesterr(opt(s, "none"), "supported")
@@ -386,6 +386,32 @@ if (length(s)) {
   nanotest(is.integer(close(s)))
 }
 
+s <- socket("bus", listen = "inproc://nanolock")
+s1 <- socket("bus", dial = "inproc://nanolock")
+nanotestz(lock(s))
+s2 <- socket("bus", dial = "inproc://nanolock")
+nanotestz(send(s, "test"))
+nanotestnn(recv(s1))
+nanotest(is_error_value(recv(s2)))
+nanotestz(unlock(s))
+nanotesterr(unlock(cv), "valid Socket")
+cv <- cv()
+pipe_notify(s, cv = cv, flag = FALSE)
+s3 <- socket("bus", dial = "inproc://nanolock")
+nanotestz(send(s, "test"))
+nanotestnn(recv(s3))
+nanotestz(lock(s, cv = cv))
+nanotesterr(lock(cv), "valid Socket")
+nanotesterr(lock(s, cv = s), "valid Condition Variable")
+s4 <- socket("bus", dial = "inproc://nanolock")
+nanotestz(send(s, "test"))
+nanotest(is_error_value(recv(s4)))
+nanotestz(close(s))
+nanotestz(close(s1))
+nanotestz(close(s2))
+nanotestz(close(s3))
+nanotestz(close(s4))
+
 fakesock <- `class<-`(new.env(), "nanoSocket")
 nanotesterr(dial(fakesock), "valid Socket")
 nanotesterr(dial(fakesock, autostart = FALSE), "valid Socket")
@@ -396,6 +422,7 @@ nanotesterr(stat(fakesock, "pipes"), "valid Socket")
 nanotesterr(close(fakesock), "valid Socket")
 nanotest(.unresolved(fakesock) == FALSE)
 fakectx <- `class<-`("test", "nanoContext")
+nanotest(.unresolved(fakectx) == FALSE)
 nanotesterr(request(fakectx, data = "test"), "valid Context")
 nanotesterr(subscribe(fakectx, NULL), "valid")
 nanotesterr(close(fakectx), "valid Context")
@@ -423,11 +450,6 @@ nanotestp(unres)
 nanotest(unresolved(unres))
 nanotest(identical(call_aio("a"), "a"))
 nanotestn(stop_aio("a"))
-nanotesterr(msg_pipe(fakesock), "valid recvAio")
-fakencurl <- `class<-`("test", "ncurlAio")
-nanotesterr(msg_pipe(fakencurl), "valid recvAio")
-fakepipe <- `class<-`("test", "nanoPipe")
-nanotesterr(close(fakepipe), "valid Pipe")
 
 nanotest(sha256("test") == "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08")
 nanotest(length(sha256(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) == 32L)
@@ -439,6 +461,8 @@ nanotest(length(sha224(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) =
 nanotest(sha224(as.raw(0L)) == "fff9292b4201617bdc4d3053fce02734166a683d7d858a7f5f59b073")
 nanotest(length(sha1("testingnoncrypto", convert = FALSE)) == 20L)
 nanotest(nchar(sha1(c(1, 2), key = "secret")) == 40L)
+nanotest(is.raw(sha1(quote(a + 2), convert = FALSE)))
+nanotest(is.character(sha256(quote(a))))
 nanotest(base64enc("test") == "dGVzdA==")
 nanotest(base64dec(base64enc("test")) == "test")
 nanotest(is.raw(base64enc(data.frame(), convert = FALSE)))
